﻿var Promise = require('promise'),
    childProcess = require('child_process'),
    spawn = require('child_process').spawn,
    diffParse = require('parse-diff'),
    logParse = require('./parse-log.js').logParse,
    util = require('util'),
    debuglog = util.debuglog ? util.debuglog('sl') : require('../alt-debug-log.js').debuglog('sl');;;

function Git(cfg) {
    this.getCfg = function getCfg() { return cfg; }
}

Git.prototype.getGitPath = function getGitPath() {
    return (this.getCfg() || {}).gitPath || 'git';
    // /tools/arch/Linux_2.6.32-x86_64/gcc-4.9.1-buildrel/bin/git
}

Git.prototype.detectCommitVersion = function detectCommitVersion(handlerParams) {
    var gitPath = this.getGitPath();
    var cfg = this.getCfg();
    var p = new Promise(function (resolve, reject) {
        
        try {
            debuglog('GIT: invoking git log -n 1.');
            debuglog('GIT PATH: '+gitPath);
            var cmd = gitPath + " log -n 1 --pretty=format:%H";
            var gitLogOutput = childProcess.exec(cmd, {
                cwd: handlerParams.buildArguments.workspacepath,
                //maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
            }, function (error, stdout, stderr) {
                //debuglog('stdout: ' + stdout);
                if (error !== null) {
                    console.error('GIT log error: ' + error);
                    console.error('stderr: ' + stderr);
                    reject(error);
                    return;
                }
                
                var commit = stdout.toString().trim();
                resolve(commit);
            });
            
        } catch (e) {
            console.error('GIT log error: ' + e);
            reject(e);
        }
    });
    return p;
}

Git.prototype.getBranchHistory = function getBranchHistory(handlerParams) {
    var gitPath = this.getGitPath();
    var cfg = this.getCfg();
    var p = new Promise(function (resolve, reject) {
        var historyLength = 100;
        if (cfg.branchHistoryLength) {
            try {
                var v = parseInt(cfg.branchHistoryLength);
                if (v < 1) {
                    throw new Error(); //will be caught below
                }
                else {
                    historyLength = v;
                }
            }
            catch (e) {
                console.error("An invalid value was specified for branchHistoryLength in sealights.json: " + cfg.branchHistoryLength + ". Valid values are positive integers. Defaulting to " + historyLength);
            }
        }
        
        try {
            
            debuglog('GIT: invoking git log to get branch history');
            debuglog('GIT PATH: ' + gitPath);
            var gitLogOutput = childProcess.exec(gitPath+' log -n ' + historyLength + ' --pretty=format:%H', {
                cwd: handlerParams.buildArguments.workspacepath,
                maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
            }, function (error, stdout, stderr) {
                if (error !== null) {
                    console.error('GIT log error: ' + error);
                    console.error('stderr: ' + stderr);
                    reject(error);
                    return;
                }
                var branchLog = logParse(stdout.toString());
                resolve(branchLog);
            });
            
        } catch (e) {
            console.error('GIT log error: ' + e);
            reject(e);
        }
    });
    return p;
}

Git.prototype.getRepositoryUrl = function getRepositoryUrl(handlerParams) {
    var gitPath = this.getGitPath();

    var p = new Promise(function (resolve, reject) {
        var gitLogOutput = childProcess.exec(gitPath + ' remote -v', {
            cwd: handlerParams.buildArguments.workspacepath,
            maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
        }, function (error, stdout, stderr) {
            if (error !== null) {
                console.error('GIT log error: ' + error);
                console.error('stderr: ' + stderr);
                reject(error);
                return;
            }

            try {
                var repositoryUrl = '';
                if (stdout.toString()) {
                    repositoryUrl = logParse(stdout.toString());
                    repositoryUrl = repositoryUrl[0].split('\t').map(function (l) {return l.trim()})[1].replace(' (fetch)', '');
                }
                resolve(repositoryUrl);
            }
            catch (e) {
                console.error('GIT log error: ' + e);
                resolve('');
            }
        });
    })
    return p;
}

module.exports = Git;